/**
 * Created by amaralj on 3/8/2016.
 *
 * Responsible for the bootstrapping of the ExpressJS plugins and routes.
 */
var express = require("express");
//var favicon = require('serve-favicon');
var logger = require('morgan');
//var cookieParser = require('cookie-parser');
var timeout = require("connect-timeout");
var bodyParser = require('body-parser');
var path = require("path");
var helmet = require("helmet");
var jwt = require('express-jwt');
var jsonwebtoken = require("jsonwebtoken");
var basicAuth = require("express-basic-auth");
var JwtHelper = require("../helpers/JwtHelper");

module.exports = function(app,config) {

// uncomment after placing your favicon in /public
//app.use(favicon(path.join(__dirname, 'public', 'favicon.ico')));
    app.use(logger('dev'));
    app.use(bodyParser.json({limit: '20mb'}));
    app.use(bodyParser.urlencoded({extended: false}));
//app.use(cookieParser());

    app.use(timeout(120000));
    app.use(haltOnTimedOut);

    function haltOnTimedOut(req, res, next) {
        if (!req.timedout) next();
    }

    app.use(express.static(path.join(__dirname, '..', '..', 'public')));

    app.use(helmet());

    app.use('/login', basicAuth({
        authorizer: myAsyncAuthorizer,
        authorizeAsync: true
    }));

    function myAsyncAuthorizer(username, password, cb) {
        try {
            var helper = new JwtHelper(config);
        } catch (err) {
            console.error(err);
            throw err;
        }
        return cb(null, helper.checkUser(username, password));
    }

    app.use('/api', jwt({
        secret: config.auth.secret,
        credentialsRequired: true,
        getToken: function fromHeaderOrQuerystring (req) {
            if (req.headers.authorization && req.headers.authorization.split(' ')[0] === 'Bearer') {
                return req.headers.authorization.split(' ')[1];
            } else if (req.query && req.query.token) {
                return req.query.token;
            }
            return null;
        }
    }).unless({ path: ['/login']}));

    app.post("/login", function(req,res){
        try {
            var ttl = Number(config.auth.token_ttl_hours);
            var token = jsonwebtoken.sign({
                exp: Math.floor(Date.now() / 1000) + (ttl * 60 * 60),
                data: 'FMC'
            }, config.auth.secret);
            res.json({
                success: true,
                message: 'Authentication successful',
                token: token
            })
        } catch (err) {
            console.error(err);
            res.json({
                success: false,
                message: 'Authentication failed',
                error: err
            })
        }

    });

    require("../../routes/api")(app,config);

// catch 404 and forward to error handler
    app.use(function (req, res, next) {
        var err = new Error('Not Found');
        err.status = 404;
        next(err);
    });

// error handlers

// development error handler
// will print stacktrace
    if (config.name !== 'prod') {
        app.use(function (err, req, res, next) {
            res.status(err.status || 500).send(err);
        });
    }

// production error handler
// no stacktraces leaked to user
    app.use(function (err, req, res, next) {
        res.status(err.status || 500).send({});
    });

    return app;

};

