/**
 * Created by John on 7/10/2017.
 *
 * A command line interface for adding ASNs and Receipts to the database
 *
 */
var program = require('commander');
var AsnHelper = require("./components/helpers/AsnHelper");
var _ = require("lodash");
var port = process.env.PORT || 8080;
var moment = require("moment");
var fs = require("fs");
var path = require("path");

var root = path.join(__dirname, "db", "perf_q");
var now = new moment();
var filename = path.join(root, "test_msgs_" + now.format("YYYYMMDDHHmmss") );

program
    .version('0.0.1')
    .option('-a, --asns', 'Generates ASNs using the Integration server')
    .option('-r, --receipts', 'Generates Receipts against some ASNs.')
    .option('-e, --endpoint [endpoint]', 'API Host (full path) to send the data.')
    .option('-c, --clinics [clinics]', 'Number of clinics for which the Objects are created.')
    .option('-b, --batch [batch]', 'Number of records per POST.')
    .option('-d, --delay [delay]', 'Number of milliseconds between POSTs.')
    .option('-u, --user [user]', 'Specify the User.  Required with EVERY option.')
    .option('-p, --password [password]', 'Specify the User\'s password.  Required with ADD and LIST options only.')
    .option('-l, --lines [lines]', 'Number of lines generated per clinic')
    .option('-f, --fileOnly', 'Writes the messages to a file and does not actually "send" them.')
    .option('-i, --inputFile [inputFile]', 'Full path. Publishes the messages in a message file. -c and -l should not be used.')
    .parse(process.argv);

var helper = new AsnHelper();
var endpoint = program.endpoint;

if (program.asns) {
    if (!program.fileOnly && (!program.user || !program.password)) {
        console.error('Both -u and -p are required!');
        process.exit(1);
    }

    if (program.inputFile) {
        if (!endpoint) endpoint = "http://localhost:" + port + "/api/v1/asn";
        helper.sendAsns(program.inputFile, program.user, program.password, program.batch, program.delay, endpoint);
    } else {

        if (!program.clinics || !program.lines) {
            console.error('Both -c and -l are required!');
            process.exit(1);
        }
        if (!program.fileOnly && (!program.batch || !program.delay)) {
            console.error('Both -b and -d are required!');
            process.exit(1);
        }
        if (!endpoint) endpoint = "http://localhost:" + port + "/api/v1/asn";

        var realFileName = initQ("asns");
        helper.generateAsns(endpoint, program.clinics, program.lines, realFileName, function(err, fn){
            closeFile(realFileName);
            if (!err && !program.fileOnly) {
                helper.sendAsns(realFileName, program.user, program.password, program.batch, program.delay, endpoint);
            }
        });

    }

} else if (program.receipts) {
    if (!program.fileOnly && (!program.batch || !program.delay)) {
        console.error('Both -b and -d are required!');
        process.exit(1);
    }

    if (program.inputFile) {
        helper.sendReceipts(program.inputFile, program.batch, program.delay, endpoint);
    } else {
        if (!program.clinics || !program.lines) {
            console.error('Both -c and -l are required!');
            process.exit(1);
        }

        if (!endpoint) endpoint = "http://localhost:3414/api/saveInventoryData";

        var realFileName = initQ("receipts");
        helper.generateReceipts(endpoint, program.clinics, program.lines, realFileName, function(err, fn){
            closeFile(realFileName);

            if (!err && !program.fileOnly) {
                helper.sendReceipts(realFileName, program.batch, program.delay, endpoint);
            }
        });

    }

}



function initQ(type) {
    var realFileName = filename + "_" + type + "_c" + program.clinics + "_l" + program.lines +  ".txt";
    if (!fs.existsSync(root)) {
        fs.mkdirSync(root);
    }
    if (!fs.existsSync(realFileName)) {
        fs.writeFileSync(realFileName, "[\n");
    }
    console.log("Using File to stage msgs for Perf...");
    return realFileName;
}

function closeFile(filename) {
    if (fs.existsSync(filename)) {
        fs.appendFileSync(filename, "\n]");
    }
}