/**
 * Created by John on 7/10/2017.
 *
 * A command line interface for adding/removing JWT users
 *
 */
var program = require('commander');
var JwtHelper = require("./components/helpers/JwtHelper");
var _ = require("lodash");

program
    .version('0.0.1')
    .option('-a, --add', 'Add a user and its password to the allowable JWT list')
    .option('-u, --user [user]', 'Specify the User.  Required with EVERY option.')
    .option('-p, --password [password]', 'Specify the User\'s password.  Required with ADD, LIST, and JOB options only.')
    .option('-d, --delete', '(Same as -r). Remove a user and its password to the allowable JWT list')
    .option('-r, --remove', '(Same as -d). Remove a user and its password to the allowable JWT list')
    .option('-l, --list', 'Lists the users in the store (requires credentials)')
    .option('-j, --job', 'Kicks off the SAP job immediately (outside of the cron schedule).  Credentials required.')
    .option('-e, --env [env]', 'Use the config file from this "environment".  If missing, will look for NODE_ENV variable. (Examples: dev, qa, test, prod, etc.)')
    .parse(process.argv);

var env =  program.env || process.env.NODE_ENV;
if (!env || env === "") {
    console.error("We need an environment to pull the API endpoint");
    process.exit(1);
} else {
    var config = require("./components/config/server." + env + ".js");
    var helper = new JwtHelper(config);

    if (program.job) {
        if (!program.user || !program.password) {
            console.error('Both -u and -p are required!');
            process.exit(1);
        } else {

            console.log("Running job for " + env);
            if (helper.checkUser(program.user, program.password)) {
                var SAP = require("./components/jobs/SAP");
                var job = new SAP(config);
                job.execute(function (err, data) {
                    if (err) {
                        console.error(err);
                        process.exit(1);
                    } else {
                        console.log("JOB finished");
                        process.exit(0);
                    }
                });
            } else {
                console.error("Invalid credentials.");
                process.exit(1);
            }

        }
    }
    else if (program.add) {
        if (!program.user || !program.password) {
            console.error('Both -u and -p are required!');
            process.exit(1);
        } else {
            helper.addUser(program.user, program.password);
        }
    } else if (program.delete || program.remove) {
        if (!program.user) {
            console.error('-u is required!');
            process.exit(1);
        } else {
            helper.removeUser(program.user);
        }
    } else if (program.list) {
        if (!program.user || !program.password) {
            console.error('Both -u and -p are required!');
            process.exit(1);
        } else {
            var users = helper.listUsers(program.user, program.password);
            _.forEach(users, function(item) {
                console.log(item);
            })
        }
    }
}

